﻿using System;
using System.Windows.Forms;
using System.Text;
using m2fz;


namespace TestFZ
{
    class Program
    {
        /// <summary>
        /// アプリケーションのメイン エントリ ポイントです。
        /// </summary>
        static mfz m_fz;

        [STAThread]
        static void Main()
        {
            try
            {
                // mfz dll Load
                m_fz = new mfz();
                // mfz dll をチェック
                if (m_fz.AreYouReady() != mfz.YES)
                {
                	MessageBox.Show("This program has not loaded FZ DLL.");
                    Application.Exit();
                    return;
                }


				// Give the commnd.
                // プロジェクトの[プロパティ][デバッグ]<コマンドライン引数>で"/a"等を設定できる
                string[] command = Environment.GetCommandLineArgs();
				string[] strategy = new string[] { "/a", "/b", "/c", "/d", "" };
				int 	 n;
				bool 	 r;

				for(n = 4; n > -1; n--)
				{
					if(command.GetUpperBound(0) == 0)
					{
						break;
					}
                    else
                    {
                        if (string.Compare(command[1], strategy[n]) == 0)
                        {
                            break;
                        }
                    }
				}

				if(n == 4)
				{
                    Headquarters dlg = new Headquarters();

                    dlg.ShowDialog();
                    n = dlg.GetStrategy(); 
                    
                    dlg.Dispose();
				}

				switch(n)
				{
				case 0:
					r = ExecuteStrategyA(strategy[n]);
					break;
				case 1:
					r = ExecuteStrategyB(strategy[n]);
					break;
				case 2:
					r = ExecuteStrategyC(strategy[n]);
					break;
				case 3:
					r = ExecuteStrategyD(strategy[n]);
					break;
				default:
					r = false;
                    break;
				}

				// mfz dll unload
                m_fz.UnLoad();

                if (r)
                {
                    Application.EnableVisualStyles();
                    Application.Run(new Form1());
                }
                else
                {
                    MessageBox.Show("The unrecognized code will terminate this program" +
									" about 10 seconds after.");
                    Application.Exit();
                }
            }
            catch (Exception e)
            {
                MessageBox.Show(e.Message);
                Application.Exit();
            }

        }

        static void ShowConditions(string command, uint magicCode, uint installer, uint hardware)
        {
            int n1 = (magicCode == 0) ? 0 : 1;
            int	n2 = (int)hardware >> 20;
            string[] id = new string[] { "Outside", "Inside" };
            string[] isInstallerUsing = new string[] { "NO", "YES" };
            string[] includingHardware = new string[] { "HDD", "CPU", "HDD and CPU" };


            MessageBox.Show(string.Format(	"Command line : {0:s}\n" +
            								"Identity : {1:s}(0x{2:x})\n" +
            								"Installer : {3:s}\n" + 
            								"Validation Code : {4:s}",
                                    		command,
                							id[n1],
                							magicCode,
                							isInstallerUsing[installer],
                                    		includingHardware[n2])	);
        }


        static bool Verify()
        {
        	//
			//// 処理内容の決定
			//
			
            // Trunk ファイル名作成
            string manufacturer = new string('\0', mfz.GTN_CHAR_SIZE);
            string productName = new string('\0', mfz.GTN_CHAR_SIZE);
            StringBuilder trunkName = new StringBuilder(null, mfz.MAX_PATH);


            manufacturer = "m.magic";	// インストーラーで使用した名前
            productName = "TestFZ";	// インストーラーで使用した名前

            m_fz.GetTrunkName(	mfz.GTN_GET_TRUNK_PATH_AND_NAME | mfz.GTN_MAKE_DIRECTORY,
                             	manufacturer,
                                productName,
                                mfz.MAX_PATH,
                                trunkName	);

            // Trunk ファイルから認証コードを読む
            bool createTrunk;
			bool getTime;
			bool putData;
			bool verify;
			byte[] time = new byte[mfz.TIME_SIZE];
			byte[] validationCode = new byte[mfz.VALDATIONCODE_SIZE];
			StringBuilder userName = new StringBuilder(null, mfz.USER_NAME_SIZE);
			int r0;


			createTrunk = getTime = putData = false;
			verify = true;

            r0 = m_fz.GetCode(trunkName.ToString(), validationCode, time, userName);

            if (r0 == mfz.COD_ERR_FILE_NOT_FOUND)
            {
                userName.Append(" ");	// スペースを入れるとログイン名になる
                getTime = putData = true;
                createTrunk = true;
                verify = false;
            }
            else if (r0 == mfz.COD_ERR_INITIAL_STATE)
            {
                userName.Append(" ");	// スペースを入れるとログイン名になる
                getTime = putData = true;
                verify = false;
            }

            // 認証コードの診断
            StringBuilder produktKey = new StringBuilder(null, mfz.PRODUCTKEY_SIZE);
            int ok = mfz.NO;
            int productKeySize = mfz.PRODUCTKEY_16;


            if (verify)
                ok = m_fz.VerifyValidationCode(userName.ToString(), validationCode);

            if (ok == mfz.NO)
            {
                // 入力されたキーをチェックします
                m_fz.ShowInputBox(	"TestFZ",
                                    "\n" +
                                    "This is the test program. " +
                    				"You must put in a signature and a key. " +
                    				"A capital letter and a small letter are distinguished.",
                    			    productKeySize,
                                    userName,
                                    produktKey	);

                // 認証コードを作る
                if (m_fz.GetValidationCode(	produktKey.ToString(),
                							userName.ToString(),
                							validationCode	) == mfz.SUCCESS)
                {
                    ok = mfz.YES;
                    putData = true;
                }
            }


            //
            //// 時間値
            //
            mfz.SYSTEMTIME systemTime = new mfz.SYSTEMTIME();
            int days;
            int r1, r2;

            // 時間値を取得する(userName は必須)
            if (getTime)
                r1 = m_fz.GetTime(userName.ToString(), time);

            systemTime.initialize();
            r2 = m_fz.GetSystemTime(userName.ToString(), time, ref systemTime);

            if (r2 == mfz.TMS_SUCCESS_AND_TIME_MADE)
                putData = true;

            days = m_fz.GetTimeSpan(userName.ToString(), time);


            //
            //// トランクファイル
            //
            int r3, r4;

            if (createTrunk)
                r3 = m_fz.CreateTrunk(trunkName.ToString(), mfz.CTK_ATTRIBUTE_READONLY);

            // Trunk file に書き込む
            if (putData)
                r4 = m_fz.PutCode(	trunkName.ToString(), 
            				        validationCode,
            						time,
            						userName.ToString()	);


            //
            //// メッセージ
            //
            string message;
			MessageBoxIcon icon;

			if (ok == mfz.NO)
            {
            	message = string.Format(	"{0:d} days have passed since you set up this program. " + 
                						    " Put in the key, if you please.\n" +
                                            "\n" +
                                            "User Name:{1:s}\n" +
                                            "DATE:{2:d}/{3:d}/{4:d} {5:d}:{6:d}:{7:d}",
                                            days,
                                            userName,
                                            systemTime.month,
                                            systemTime.day,
                                            systemTime.year,
                                            systemTime.hour,
                                            systemTime.minute,
                                            systemTime.second	);

				icon = MessageBoxIcon.Warning;
			}
			else
			{
				message = string.Format(	"Validation code is enabled for this program. \n" + 
				    						"\n" +
                	    					"User Name:{1:s}\n" +
                                            "Installation date : {2:d}/{3:d}/{4:d} {5:d}:{6:d}:{7:d}" +
                                            "Time span : {0:d}",
                                            days,
                                            userName,
                                            systemTime.month,
                                            systemTime.day,
                                            systemTime.year,
                                            systemTime.hour,
                                            systemTime.minute,
                                            systemTime.second	);

				icon = MessageBoxIcon.Information;
     		}

     		 MessageBox.Show(message, "TestFZ", MessageBoxButtons.OK, icon);

             return ok == mfz.YES;
        }

        static bool ExecuteStrategyD(string command)
        {
            //  条件：
            //	Ａ．ＩＤは、実行ファイルに内包
            //	Ｂ．Trunk ファイルを使用。（認証コード使用）	
            //
            //	動作：
            //	１．プログラム起動時にプロダクトキーを確認する
            //	２．認証コードは、Trunk ファイルに書き込む
            //	３．起動ごとに認証コードを確認する
            //	
            //	このサンプルはこのままでは動作しません。適切なＩＤ等をセットしてください。

            /*	MANUFACUTURER: m.magic
            PRODUCT NAME: TestFZ
            STORING PLACE OF IDENTITY: Inside
            INCLUDED HARDWARE: CPU
            VSI: Ready
            TOOL: Visual C++
            MAGIC CODE(x86): 0xd87f80c8
            MAGIC CODE(x64): 0xa883ad35
            NOTES: 認証コードＣＰＵのみ
	
            Product Key
            0	01BbOwYqUTzX01al
            1	01QR698qWGYhaGcX
            2	01SotPmXt1tE8uzI
            3	01YAFRk9LMDZnlt9*/

            // Application Identity
            byte[] id = new byte[192] {
	        0x18,0x4C,0x06,0x71,0xBA,0x36,0x7B,0xE0,0xA3,0xE4,0x61,0xAA,0xA3,0xE4,0x61,0xAA,0x54,0x47,0x25,0x8B,
	        0xEA,0x44,0x2A,0xCA,0xFA,0xF5,0xE4,0x70,0x99,0x60,0xA7,0xA9,0x6F,0x91,0x99,0xF2,0x3E,0x1A,0xB6,0x69,
	        0xD2,0x48,0x1F,0x56,0xDD,0x40,0xD9,0xE1,0xAF,0x71,0xFF,0x34,0xB2,0xAA,0x0C,0x7E,0x77,0xA0,0xE2,0x58,
	        0x03,0x6D,0xE1,0x25,0x25,0xD8,0xF9,0x1A,0x39,0xAA,0x80,0x17,0x1B,0x88,0x3E,0x3C,0x23,0xFA,0xC9,0x25,
	        0x3D,0xA3,0xBD,0x32,0x29,0x79,0x78,0x17,0x33,0x5E,0xC2,0x14,0x03,0x63,0xA1,0x25,0x15,0x5D,0xB1,0x0A,
	        0x19,0x79,0xE0,0x37,0x0B,0xED,0x16,0x2C,0x23,0x07,0x19,0x25,0x2D,0xC5,0xF5,0x22,0x29,0x5B,0xE8,0x37,
	        0x23,0x3D,0x9A,0x04,0x03,0xD9,0x61,0x25,0x05,0x59,0x69,0x3A,0x39,0x8A,0x40,0x17,0x3B,0x52,0x6E,0x1C,
	        0x23,0x85,0x49,0x25,0x1D,0xD0,0x2D,0x12,0x29,0x5F,0x58,0x17,0x13,0x39,0x72,0x34,0x03,0x96,0xE1,0x25,
	        0x35,0xCD,0xE1,0x2A,0x19,0x12,0xE0,0x37,0x2B,0xDE,0x86,0x0C,0x23,0x47,0xB9,0x25,0x0D,0xB7,0x65,0x02,
	        0x29,0xB9,0xA8,0x37,0x03,0xBF,0x4A,0x24,0x03,0xBF,0x61,0x25
	        };
	        
	        // ＤＬＬを初期化する
            mfz.INI_PARAMS_A ini = new mfz.INI_PARAMS_A();

            ini.initialize();
            ini.functions = mfz.INI_STANDARD | mfz.INI_VALDATIONCODE_HDD_AND_CPU;
            //mfz.INI_STANDARD;//mfz.INI_LEGACY;//mfz.INI_PRODUCTKEY_VERSION_40;
            ini.size = mfz.INI_PARAMS_A_SIZE;
            id.CopyTo(ini.identity, 0);

            ShowConditions(command, 0, mfz.NO, ini.functions & mfz.INI_VALDATIONCODE);

            if (m_fz.InitializeA(ref ini) != mfz.SUCCESS)
                return false;
            else
                return Verify();
        }

        static bool ExecuteStrategyC(string command)
        {
            //  条件：
            //	Ａ．ＩＤは、実行ファイルに内包
            //	Ｂ．Trunk ファイルを使用。（認証コード使用）	
            //
            //	動作：
            //	１．プログラム起動時にプロダクトキーを確認する
            //	２．認証コードは、Trunk ファイルに書き込む
            //	３．起動ごとに認証コードを確認する
            //	
            //	このサンプルはこのままでは動作しません。適切なＩＤ等をセットしてください。

            /*	MANUFACUTURER: m.magic
            PRODUCT NAME: TestFZ
            STORING PLACE OF IDENTITY: Inside
            INCLUDED HARDWARE: CPU
            VSI: Ready
            TOOL: Visual C++
            MAGIC CODE(x86): 0xd87f80c8
            MAGIC CODE(x64): 0xa883ad35
            NOTES: 認証コードＣＰＵのみ
	
            Product Key
            0	01BbOwYqUTzX01al
            1	01QR698qWGYhaGcX
            2	01SotPmXt1tE8uzI
            3	01YAFRk9LMDZnlt9*/

            // Application Identity
            byte[] id = new byte[192] {
	        0x18,0x4C,0x06,0x71,0xBA,0x36,0x7B,0xE0,0xA3,0xE4,0x61,0xAA,0xA3,0xE4,0x61,0xAA,0x54,0x47,0x25,0x8B,
	        0xEA,0x44,0x2A,0xCA,0xFA,0xF5,0xE4,0x70,0x99,0x60,0xA7,0xA9,0x6F,0x91,0x99,0xF2,0x3E,0x1A,0xB6,0x69,
	        0xD2,0x48,0x1F,0x56,0xDD,0x40,0xD9,0xE1,0xAF,0x71,0xFF,0x34,0xB2,0xAA,0x0C,0x7E,0x77,0xA0,0xE2,0x58,
	        0x03,0x6D,0xE1,0x25,0x25,0xD8,0xF9,0x1A,0x39,0xAA,0x80,0x17,0x1B,0x88,0x3E,0x3C,0x23,0xFA,0xC9,0x25,
	        0x3D,0xA3,0xBD,0x32,0x29,0x79,0x78,0x17,0x33,0x5E,0xC2,0x14,0x03,0x63,0xA1,0x25,0x15,0x5D,0xB1,0x0A,
	        0x19,0x79,0xE0,0x37,0x0B,0xED,0x16,0x2C,0x23,0x07,0x19,0x25,0x2D,0xC5,0xF5,0x22,0x29,0x5B,0xE8,0x37,
	        0x23,0x3D,0x9A,0x04,0x03,0xD9,0x61,0x25,0x05,0x59,0x69,0x3A,0x39,0x8A,0x40,0x17,0x3B,0x52,0x6E,0x1C,
	        0x23,0x85,0x49,0x25,0x1D,0xD0,0x2D,0x12,0x29,0x5F,0x58,0x17,0x13,0x39,0x72,0x34,0x03,0x96,0xE1,0x25,
	        0x35,0xCD,0xE1,0x2A,0x19,0x12,0xE0,0x37,0x2B,0xDE,0x86,0x0C,0x23,0x47,0xB9,0x25,0x0D,0xB7,0x65,0x02,
	        0x29,0xB9,0xA8,0x37,0x03,0xBF,0x4A,0x24,0x03,0xBF,0x61,0x25
	        };

            // ＤＬＬを初期化する
            mfz.INI_PARAMS_A ini = new mfz.INI_PARAMS_A();

            ini.initialize();
            ini.functions = mfz.INI_STANDARD;
            //mfz.INI_STANDARD;//mfz.INI_LEGACY;//mfz.INI_PRODUCTKEY_VERSION_40;
            ini.size = mfz.INI_PARAMS_A_SIZE;
            id.CopyTo(ini.identity, 0);
            

            ShowConditions(command, 0, mfz.NO, ini.functions & mfz.INI_VALDATIONCODE);

            if (m_fz.InitializeA(ref ini) != mfz.SUCCESS)
                return false;
            else
                return Verify();
        }

        static bool ExecuteStrategyB(string command)
        {
            //  条件：
            //	Ａ．ＩＤは、ＤＬＬ(Magiccode使用)に内包
            //	Ｂ．Trunk ファイルを使用。（認証コード使用）	
            //
            //	動作：
            //	１．起動時に、プロダクトキーを入力させる
            //	２．認証コードを所定の Trunk ファイルに書き込む
            //	３．以後、起動ごとに認証コードを確認する
            //	
            //	このサンプルはこのままでは動作しません。適切なＩＤ等をセットしてください。

            /*	MANUFACUTURER: m.magic
            PRODUCT NAME: TestFZ
            STORING PLACE OF IDENTITY: Inside
            INCLUDED HARDWARE: CPU
            VSI: Ready
            TOOL: Visual C++
            MAGIC CODE(x86): 0xd87f80c8
            MAGIC CODE(x64): 0xa883ad35
            NOTES: 認証コードＣＰＵのみ
	
            // Application Identity
            const unsigned char id[192] = {
            0x18,0x4C,0x06,0x71,0xBA,0x36,0x7B,0xE0,0xA3,0xE4,0x61,0xAA,0xA3,0xE4,0x61,0xAA,0x54,0x47,0x25,0x8B,
            0xEA,0x44,0x2A,0xCA,0xFA,0xF5,0xE4,0x70,0x99,0x60,0xA7,0xA9,0x6F,0x91,0x99,0xF2,0x3E,0x1A,0xB6,0x69,
            0xD2,0x48,0x1F,0x56,0xDD,0x40,0xD9,0xE1,0xAF,0x71,0xFF,0x34,0xB2,0xAA,0x0C,0x7E,0x77,0xA0,0xE2,0x58,
            0x03,0x6D,0xE1,0x25,0x25,0xD8,0xF9,0x1A,0x39,0xAA,0x80,0x17,0x1B,0x88,0x3E,0x3C,0x23,0xFA,0xC9,0x25,
            0x3D,0xA3,0xBD,0x32,0x29,0x79,0x78,0x17,0x33,0x5E,0xC2,0x14,0x03,0x63,0xA1,0x25,0x15,0x5D,0xB1,0x0A,
            0x19,0x79,0xE0,0x37,0x0B,0xED,0x16,0x2C,0x23,0x07,0x19,0x25,0x2D,0xC5,0xF5,0x22,0x29,0x5B,0xE8,0x37,
            0x23,0x3D,0x9A,0x04,0x03,0xD9,0x61,0x25,0x05,0x59,0x69,0x3A,0x39,0x8A,0x40,0x17,0x3B,0x52,0x6E,0x1C,
            0x23,0x85,0x49,0x25,0x1D,0xD0,0x2D,0x12,0x29,0x5F,0x58,0x17,0x13,0x39,0x72,0x34,0x03,0x96,0xE1,0x25,
            0x35,0xCD,0xE1,0x2A,0x19,0x12,0xE0,0x37,0x2B,0xDE,0x86,0x0C,0x23,0x47,0xB9,0x25,0x0D,0xB7,0x65,0x02,
            0x29,0xB9,0xA8,0x37,0x03,0xBF,0x4A,0x24,0x03,0xBF,0x61,0x25
            };
	
            Product Key
            0	01BbOwYqUTzX01al
            1	01QR698qWGYhaGcX
            2	01SotPmXt1tE8uzI
            3	01YAFRk9LMDZnlt9*/

            // ＤＬＬを初期化する
            mfz.INI_PARAMS_B ini = new mfz.INI_PARAMS_B();

            ini.initialize();
            ini.functions = mfz.INI_STANDARD;//mfz.INI_LEGACY;//mfz.INI_PRODUCTKEY_VERSION_40;
            ini.size = mfz.INI_PARAMS_B_SIZE;
            // 必須！！
#if AMD64
	        ini.magicCode = 0xa883ad35;
#else
            ini.magicCode = 0xd87f80c8;
#endif

            ShowConditions(command, ini.magicCode, mfz.NO, mfz.INI_VALDATIONCODE_CPU);

            if (m_fz.InitializeB(ref ini) != mfz.SUCCESS)
                return false;
            else
                return Verify();
        }

        static bool ExecuteStrategyA(string command)
        {
            //  条件：
            //	Ａ．ＩＤは、インストーラーで使用したＤＬＬと実行ファイルに内包
            //	Ｂ．Trunk ファイルを使用。（認証コード使用）	
            //
            //	動作：
            //	１．インストーラーによりプロダクトキーは認証済み
            //	２．認証コードは、インストーラーにより所定の Trunk ファイルに書き込み済み
            //	３．起動ごとに認証コードを確認する
            //	
            //	このサンプルはこのままでは動作しません。適切なＩＤ等をセットしてください。

            /*	MANUFACUTURER: m.magic
            PRODUCT NAME: TestFZ
            STORING PLACE OF IDENTITY: Inside
            INCLUDED HARDWARE: CPU
            VSI: Ready
            TOOL: Visual C++
            MAGIC CODE(x86): 0xd87f80c8
            MAGIC CODE(x64): 0xa883ad35
            NOTES: 認証コードＣＰＵのみ
	
            Product Key
            0	01BbOwYqUTzX01al
            1	01QR698qWGYhaGcX
            2	01SotPmXt1tE8uzI
            3	01YAFRk9LMDZnlt9*/


            // Application Identity
            byte[] id = new byte[192] {
	        0x18,0x4C,0x06,0x71,0xBA,0x36,0x7B,0xE0,0xA3,0xE4,0x61,0xAA,0xA3,0xE4,0x61,0xAA,0x54,0x47,0x25,0x8B,
	        0xEA,0x44,0x2A,0xCA,0xFA,0xF5,0xE4,0x70,0x99,0x60,0xA7,0xA9,0x6F,0x91,0x99,0xF2,0x3E,0x1A,0xB6,0x69,
	        0xD2,0x48,0x1F,0x56,0xDD,0x40,0xD9,0xE1,0xAF,0x71,0xFF,0x34,0xB2,0xAA,0x0C,0x7E,0x77,0xA0,0xE2,0x58,
	        0x03,0x6D,0xE1,0x25,0x25,0xD8,0xF9,0x1A,0x39,0xAA,0x80,0x17,0x1B,0x88,0x3E,0x3C,0x23,0xFA,0xC9,0x25,
	        0x3D,0xA3,0xBD,0x32,0x29,0x79,0x78,0x17,0x33,0x5E,0xC2,0x14,0x03,0x63,0xA1,0x25,0x15,0x5D,0xB1,0x0A,
	        0x19,0x79,0xE0,0x37,0x0B,0xED,0x16,0x2C,0x23,0x07,0x19,0x25,0x2D,0xC5,0xF5,0x22,0x29,0x5B,0xE8,0x37,
	        0x23,0x3D,0x9A,0x04,0x03,0xD9,0x61,0x25,0x05,0x59,0x69,0x3A,0x39,0x8A,0x40,0x17,0x3B,0x52,0x6E,0x1C,
	        0x23,0x85,0x49,0x25,0x1D,0xD0,0x2D,0x12,0x29,0x5F,0x58,0x17,0x13,0x39,0x72,0x34,0x03,0x96,0xE1,0x25,
	        0x35,0xCD,0xE1,0x2A,0x19,0x12,0xE0,0x37,0x2B,0xDE,0x86,0x0C,0x23,0x47,0xB9,0x25,0x0D,0xB7,0x65,0x02,
	        0x29,0xB9,0xA8,0x37,0x03,0xBF,0x4A,0x24,0x03,0xBF,0x61,0x25
	        };

            // ＤＬＬを初期化する
            mfz.INI_PARAMS_A ini = new mfz.INI_PARAMS_A();

            ini.initialize();
            ini.functions = mfz.INI_STANDARD | mfz.INI_VALDATIONCODE_CPU;
            //mfz.INI_STANDARD;//mfz.INI_LEGACY;//mfz.INI_PRODUCTKEY_VERSION_40;
            ini.size = mfz.INI_PARAMS_A_SIZE;
            id.CopyTo(ini.identity, 0);

            ShowConditions(command, 0, mfz.YES, ini.functions & mfz.INI_VALDATIONCODE);

            if (m_fz.InitializeA(ref ini) != mfz.SUCCESS)
                return false;
            else
                return Verify();
        }
    }
}
